import 'dart:async';

import 'package:flutter/widgets.dart';
import 'package:rxdart/rxdart.dart';
import 'package:rxdart/subjects.dart';

/// Takes in a "dependency" stream and a function that returns a "source" stream
/// given a dependency value. Listens to the dependency and every time a new
/// value is emitted, it asks the source builder to provide a new source stream.
/// Then emits through [stream] the values emitted by the source stream.
///
/// This is for things like fetching data depending on a certain value, such as
/// a filter value.
///
/// Example:
/// ```dart
/// depStream.dependOn(
///   dependency: showArchived,
///   initialSource: _repo.getActiveNotes(),
///   sourceBuilder: (showArchived) =>
///       showArchived ? _repo.getArchivedNotes() : _repo.getActiveNotes(),
/// );
///
/// // ...
///
/// depStream.listen((event) {
///   // Do something...
/// });
/// ```
class DependantStream<S, T> {
  final _output = BehaviorSubject<T>();

  Stream<T> _source;
  StreamSubscription _sourceSub;
  StreamSubscription _dependencySub;

  Stream<T> get stream => _output.stream;

  void dependOn({
    @required Stream<S> dependency,
    @required Stream<T> initialSource,
    @required SourceBuilder<S, T> sourceBuilder,
  }) {
    _reassignSource(initialSource);
    _dependencySub?.cancel();
    _dependencySub = dependency.listen((data) {
      _reassignSource(sourceBuilder(data));
    });
  }

  StreamSubscription<T> listen(void onData(T event),
      {Function onError, void onDone(), bool cancelOnError}) {
    return stream.listen(
      onData,
      onError: onError,
      onDone: onDone,
      cancelOnError: cancelOnError,
    );
  }

  void _reassignSource(Stream<T> source) {
    _source = source;
    _sourceSub?.cancel();
    _sourceSub = _source.listen((data) {
      _output.sink.add(data);
    });
  }

  void dispose() {
    _output.close();
  }
}

typedef Stream<T> SourceBuilder<S, T>(S);
