import 'package:arch_foundations/mixin/snackbarer.dart';
import 'package:flutter/material.dart';

/// A mixin that takes care of UI logic for [NetworkBoundResource]s.
///
/// If given [promptLoading] and there is data in loading state, instead
/// [loadingBuilder], [contentBuilder] and [promptLoading] will be called. This
/// allows you to show cached data and inform the user that new data is being
/// loaded.
///
/// If given [promptError] and there is data in error state, instead
/// [errorBuilder], [contentBuilder] and [promptError] will be called. This
/// allows you to show cached data and inform the user that new data wasn't
/// fetched.
mixin NetworkBoundResourceClient implements SnackBarEr {
  AsyncWidgetBuilder buildForStream<T>({
    @required Function(T) contentBuilder,
    @required Function(String) errorBuilder,
    @required Function loadingBuilder,
    @required Function emptyBuilder,
    Function promptLoading,
    Function(String) promptError,
  }) {
    return (context, snapshot) {
      final res = snapshot.data;
      if (snapshot.hasError) {
        return errorBuilder(snapshot.error.toString());
      }
      if (snapshot.hasData) {
        if (res.isLoading()) {
          if (res.data != null && promptLoading != null) {
            promptLoading();
            return contentBuilder(res.data);
          } else {
            return loadingBuilder();
          }
        } else if (res.isEmpty()) {
          return emptyBuilder();
        } else if (res.isError()) {
          if (res.data != null && promptError != null) {
            promptError(res.message);
            return contentBuilder(res.data);
          } else {
            return errorBuilder(res.message);
          }
        } else if (res.data != null) {
          return contentBuilder(res.data);
        } else {
          return Text('!?');
        }
      }
      return Text('!?');
    };
  }
}
